function settings = rainSTORM_default_settings_loc_algos(varargin)

if isempty(varargin)
    settings.names = {...
        'Least-Squares Multi Gaussian 2D linear Bg';...
        'Least-Squares Multi Gaussian 2D const Bg';...
        'Least-Squares Gaussian 2D linear Bg';...
        'Least-Squares Gaussian 2D const Bg';...
        'Least-Squares Gaussian Halt3';...
        'Least-Squares Gaussian Thorough';...
        'Centre of Mass';...
        'Test Algo'};
    settings.fhandle = {...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocMultiGF_2D_linearBg(frameIdx, myFrame,myPixels,bkgdSig,settings);...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocMultiGF_2D_constBg(frameIdx, myFrame,myPixels,bkgdSig,settings);...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocGF_2D_linearBg(frameIdx, myFrame,myPixels,bkgdSig,settings);...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocGF_2D_constBg(frameIdx, myFrame,myPixels,bkgdSig,settings);...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocGF3(frameIdx, myFrame,myPixels,bkgdSig,settings);...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocGF(frameIdx, myFrame,myPixels,bkgdSig,settings);...
        @(frameIdx, myFrame,myPixels,rad,bkgdSig)rainSTORM_fitCoM(frameIdx, myFrame,myPixels,rad,bkgdSig);...
        @(frameIdx, myFrame,myPixels,bkgdSig,settings)rainSTORM_fitLocGF3(frameIdx, myFrame,myPixels,bkgdSig,settings)};
    return
end

loc_algo_name = varargin{1};
switch loc_algo_name
    case {  'Least-Squares Multi Gaussian 2D linear Bg';...
            'Least-Squares Multi Gaussian 2D const Bg'}
        settings = gen_struct_2;            
    case {  'Least-Squares Gaussian 2D linear Bg';...
            'Least-Squares Gaussian 2D const Bg';...
            'Least-Squares Gaussian Halt3';...
            'Least-Squares Gaussian Thorough';...
            'Centre of Mass'}
        settings = gen_struct_1;
    case 'Test Algo'
        settings = gen_struct_3;
    otherwise
        error(['Unknown algo_name: ', algo_name])
end

end

function settings = gen_struct_1()
settings=[];
settings.ImageFilter = struct(...
    'val','Average',...
    'string',{{'none', 'Average', 'Laplacian of Gaussian' ,'Determinant of Hessian' }},...
    'style','popup',...
    'text','Image filter algorithm',...
    'tooltip','Filter applied before candidate selection',...
    'type','text');
settings.Thresh = struct(...
    'val',0,...
    'text','ROI Candidate Threshold',...
    'tooltip','Threshold to filter out ROI candidates with low signal. [Raw Brightness, (sum of 9 pix val)]',...
    'type','number');
settings.Thresh2 = struct(...
    'val',0,...
    'text','ROI Candidate Relative Peak Value Threshold',...
    'tooltip','Threshold ROI candidates with low peak value. Candidate will pass through if (peak value - min(roi)) > thresh*std(background)',...
    'type','number');
settings.allowSig = struct(...
    'val',[0.5 3+1],...
    'text','Allowed Sigma [pix]',...
    'tooltip','Stop ROI fitting if iteration goes out of this range [min max]. Check: [0.5 (Radius of ROI+1)] [Raw Pixel]',...
    'type','number');
settings.allowX = struct(...
    'val',2,...
    'text','Allowed  Distance [pix]',...
    'tooltip','Stop ROI fitting if iteration goes further than this value. [Raw Pixel]',...
    'type','number');
settings.initSig = struct(...
    'val',1.3,...
    'text','Initial guess of PSF Sigma [pix]',...
    'tooltip','The initial sigma value of PSF fitting. [Raw Pixel]',...
    'type','number');
settings.maxIts = struct(...
    'val',6,...
    'text','Maximum Iterations',...
    'tooltip','The maximum number of iteretions of ROI fitting. [-]',...
    'type','number');
settings.prevSF = struct(...
    'val',5,...
    'text','Preview Scale Factor [int]',...
    'tooltip','Scale factor to render sum image, to have the same size as the super resolved image. [integer]',...
    'type','number');
settings.rad = struct(...
    'val',3,...
    'text','Radius of ROI [pix]',...
    'tooltip','''Radius of ROI'' = 3 means that the area of interest for each spot is a 7x7 area. [Raw Pixel]',...
    'type','number');
settings.tol = struct(...
    'val',0.5,...
    'text','Tolerance [-]',...
    'tooltip','Flag the fit as acceptable and save the fit result if the residue is below this value. [-]',...
    'type','number');
end

function settings = gen_struct_2()
settings=[];
settings.ImageFilter = struct(...
    'val','none',...
    'string',{{'none', 'Average', 'Laplacian of Gaussian' ,'Determinant of Hessian' }},...
    'style','popup',...
    'text','Image filter algorithm',...
    'tooltip','Filter applied before candidate selection',...
    'type','text');
settings.Thresh = struct(...
    'val',0,...
    'text','ROI Candidate Threshold',...
    'tooltip','Threshold to filter out ROI candidates with low signal. [Raw Brightness, (sum of 9 pix val)]',...
    'type','number');
settings.Thresh2 = struct(...
    'val',4,...
    'text','ROI Candidate Relative Peak Value Threshold',...
    'tooltip','Threshold ROI candidates with low peak value. Candidate will pass through if (peak value - min(roi)) > thresh*std(background)',...
    'type','number');
settings.allowSig = struct(...
    'val',[0.5 3+1],...
    'text','Allowed Sigma [pix]',...
    'tooltip','Stop ROI fitting if iteration goes out of this range [min max]. Check: [0.5 (Radius of ROI+1)] [Raw Pixel]',...
    'type','number');
settings.allowX = struct(...
    'val',2,...
    'text','Allowed  Distance [pix]',...
    'tooltip','Stop ROI fitting if iteration goes further than this value. [Raw Pixel]',...
    'type','number');
settings.initSig = struct(...
    'val',1.3,...
    'text','Initial guess of PSF Sigma [pix]',...
    'tooltip','The initial sigma value of PSF fitting. [Raw Pixel]',...
    'type','number');
settings.maxIts = struct(...
    'val',6,...
    'text','Maximum Iterations',...
    'tooltip','The maximum number of iteretions of ROI fitting. [-]',...
    'type','number');
settings.prevSF = struct(...
    'val',5,...
    'text','Preview Scale Factor [int]',...
    'tooltip','Scale factor to render sum image, to have the same size as the super resolved image. [integer]',...
    'type','number');
settings.rad = struct(...
    'val',3,...
    'text','Radius of ROI [pix]',...
    'tooltip','''Radius of ROI'' = 3 means that the area of interest for each spot is a 7x7 area. [Raw Pixel]',...
    'type','number');
settings.tol = struct(...
    'val',0.5,...
    'text','Tolerance [-]',...
    'tooltip','Flag the fit as acceptable and save the fit result if the residue is below this value. [-]',...
    'type','number');
end




function settings = gen_struct_3()

settings=[];
settings.Thresh = struct(...
    'val',0,...
    'text','ROI Candidate Threshold',...
    'tooltip','Threshold to filter out ROI candidates with low signal. [Raw Brightness, (sum of 9 pix val)]',...
    'type','number');
settings.allowSig = struct(...
    'val',[0.5 3+1],...
    'text','Allowed Sigma [pix]',...
    'tooltip','Stop ROI fitting if iteration goes out of this range [min max]. Check: [0.5 (Radius of ROI+1)] [Raw Pixel]',...
    'type','number');
settings.allowX = struct(...
    'val',2,...
    'text','Allowed  Distance [pix]',...
    'tooltip','Stop ROI fitting if iteration goes further than this value. [Raw Pixel]',...
    'type','number');
settings.initSig = struct(...
    'val',1.3,...
    'text','Initial guess of PSF Sigma [pix]',...
    'tooltip','The initial sigma value of PSF fitting. [Raw Pixel]',...
    'type','number');
settings.maxIts = struct(...
    'val',6,...
    'text','Maximum Iterations',...
    'tooltip','The maximum number of iteretions of ROI fitting. [-]',...
    'type','number');
settings.prevSF = struct(...
    'val',5,...
    'text','Preview Scale Factor [int]',...
    'tooltip','Scale factor to render sum image, to have the same size as the super resolved image. [integer]',...
    'type','number');
settings.rad = struct(...
    'val',3,...
    'text','Radius of ROI [pix]',...
    'tooltip','''Radius of ROI'' = 3 means that the area of interest for each spot is a 7x7 area. [Raw Pixel]',...
    'type','number');
settings.tol = struct(...
    'val',0.5,...
    'text','Tolerance [-]',...
    'tooltip','Flag the fit as acceptable and save the fit result if the residue is below this value. [-]',...
    'type','number');

for idx_param = 1:5
    settings.(['Test',num2str(idx_param,'%2.2i')]) = struct(...
        'val',idx_param,...
        'text',['Test',num2str(idx_param,'%2.2i'),' [-]'],...
        'tooltip',['Test ',num2str(idx_param),' this value. [-]'],...
        'type','number');
end

end
