function settings = rainSTORM_default_settings_recon_algo(varargin)

if isempty(varargin)
    settings.names = {...
        'Simple Histogram';...
        'Jittered Histogram';...
        'Average Precision Image';...
        'HSV confidence image';...
        'Intensity Precision Image';...
        'Trajectory Fitting';...
        'Test Recon Algo'};
    settings.fhandle = {...
        @(SupResParams,settings)rainSTORM_recon(SupResParams,settings);...
        @(SupResParams,settings)rainSTORM_recon_JH(SupResParams,settings);...
        @(SupResParams,settings)rainSTORM_reconAVP(SupResParams,settings);...
        @(SupResParams,settings)rainSTORM_reconHSV(SupResParams,settings);...
        @(SupResParams,settings)rainSTORM_reconIP(SupResParams,settings);...
        @(SupResParams,settings)rainSTORM_recon_Trajectory(SupResParams,settings);...
        @(SupResParams,settings)rainSTORM_recon_test_algo(SupResParams,settings)};
    return
end

recon_algo_name = varargin{1};
switch recon_algo_name
    case 'Simple Histogram'
        settings = gen_struct_simple_hist;
    case 'Jittered Histogram'
        settings = gen_struct_JH;
    case 'Average Precision Image'
        settings = gen_struct_simple_hist;
    case 'HSV confidence image'
        settings = gen_struct_simple_hist;
    case 'Intensity Precision Image'
        settings = gen_struct_simple_hist;
    case 'Trajectory Fitting'
        settings = gen_struct_Trajectory_Fitting;
    case 'Test Recon Algo'
        settings = gen_struct_test;
    otherwise
        error(['Unknown Recon Algo Name: ', recon_algo_name])
end

end

function settings = gen_struct_simple_hist()
settings=[];
settings.linMag = struct(...
    'val',8,...
    'text','Reconstruction Scale Factor',...
    'tooltip','Create this many super-res rows/cols per CCD row/column. [-]',...
    'type','number');
settings.sizeOfCCDFrame = struct(...
    'val',[128 128],...
    'text','Size of frame',...
    'tooltip','The size of the raw image, this shall be overwritten according to the raw data. [pix]',...
    'type','number');
settings.pixelWidth = struct(...
    'val',160,...
    'text','Size of raw image pixel. [nm]',...
    'tooltip','The size of the raw image pixel, this shall be overwritten according to the raw data. [nm]',...
    'type','number');
end

function settings = gen_struct_JH()
settings=[];
settings.jhLinMag_max = struct(...
    'val',8,...
    'text','Max. Recon. Scale Factor',...
    'tooltip','Create this many super-res rows/cols per CCD row/column maximum. [-]',...
    'type','number');
settings.jhAlpha = struct(...
    'val',0.5,...
    'text','jhAlpha',...
    'tooltip','See [Silverman, page 101 onwards]',...
    'type','number');
settings.jhArbitFactor = struct(...
    'val',0.85,...
    'text','jhArbitFactor',...
    'tooltip','For tweaking the jittering (kernel) width. Try 1',...
    'type','number');
settings.jhFinesse = struct(...
    'val',0.5,...
    'text','jhFinesse',...
    'tooltip','Tweaks the pixel width of reconstruction. Try 1',...
    'type','number');
settings.jhNJitters = struct(...
    'val',60,...
    'text','jhNJitters',...
    'tooltip','Number of Jitters per localisation.',...
    'type','number');
settings.jhPilotCoarseness = struct(...
    'val',1,...
    'text','jhPilotCoarseness',...
    'tooltip','To coarsen pilot estimate.',...
    'type','number');
settings.flagJHBlur = struct(...
    'val',1,...
    'text','flagJHBlur',...
    'tooltip',' To blur pilot estimate by convolution w/ ones(3).',...
    'type','number');
settings.sizeOfCCDFrame = struct(...
    'val',[128 128],...
    'text','Size of frame',...
    'tooltip','The size of the raw image, this shall be overwritten according to the raw data. [pix]',...
    'type','number');
settings.pixelWidth = struct(...
    'val',160,...
    'text','Size of raw image pixel. [nm]',...
    'tooltip','The size of the raw image pixel, this shall be overwritten according to the raw data. [nm]',...
    'type','number');
end


function settings = gen_struct_Trajectory_Fitting()
settings=[];
settings.linMag = struct(...
    'val',8,...
    'text','Reconstruction Scale Factor',...
    'tooltip','Create this many super-res rows/cols per CCD row/column. [-]',...
    'type','number');
settings.r_acc = struct(...
    'val',150,...
    'inverted',0,...
    'text','Acceptance Radius',...
    'tooltip','The max distance of two localizations to consider them in the same trajectory. [nm]',...
    'type','number');
settings.x_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','X coordinate',...
    'tooltip','The range of localization coordinates in the x direction. [pix]',...
    'type','number');
settings.y_coord = struct(...
    'val',[-Inf Inf],...
    'inverted',0,...
    'text','Y coordinate',...
    'tooltip','The range of localization coordinates in the x direction. [pix]',...
    'type','number');
settings.sizeOfCCDFrame = struct(...
    'val',[128 128],...
    'text','Size of frame',...
    'tooltip','The size of the raw image, this shall be overwritten according to the raw data. [pix]',...
    'type','number');
settings.pixelWidth = struct(...
    'val',160,...
    'text','Size of raw image pixel. [nm]',...
    'tooltip','The size of the raw image pixel, this shall be overwritten according to the raw data. [nm]',...
    'type','number');
end


function settings = gen_struct_test()
settings=[];
settings.linMag = struct(...
    'val',5,...
    'text','Reconstruction Scale Factor',...
    'tooltip','Create this many super-res rows/cols per CCD row/column. [-]',...
    'type','number');
settings.sizeOfCCDFrame = struct(...
    'val',[128 128],...
    'text','Size of frame',...
    'tooltip','The size of the raw image, this shall be overwritten according to the raw data. [pix]',...
    'type','number');
settings.pixelWidth = struct(...
    'val',160,...
    'text','Size of raw image pixel. [nm]',...
    'tooltip','The size of the raw image pixel, this shall be overwritten according to the raw data. [nm]',...
    'type','number');
for idx_param = 1:11
    settings.(['Test',num2str(idx_param,'%2.2i')]) = struct(...
        'val',idx_param,...
        'text',['Test',num2str(idx_param,'%2.2i'),' [-]'],...
        'tooltip',['Test ',num2str(idx_param),' this value. [-]'],...
        'type','number');
end
end