function rainSTORM_histogramTracker(reviewer,figNewReconHandle)
% 	Summary:	
%	Version:	0.1 - initial
%	Note:		http://devopsreactions.tumblr.com/post/75139222312/looking-at-someone-elses-code
%	Package:	rainSTORM v3.1+    (based on Box Tracker and Bridge Tracker and Histograms)
%	Developer:	gajdost
%	Name:		Gajdos Tamás
%	Company:	University of Szeged

% FUNCTION
%   Selects localizations within a user-defined box
%   Creates "the histogram" from the area
%
% USER NOTES
% 0. This tool identifies localizations within a user-defined rectangle.
% 1. Apply this tool to the most recent super-res reconstruction Figure
%    It searches for reviewed Posits - i.e. accepted Localisations -
%    which are only retained for one Figure at a time
%    To avoid confusion, I recommend closing all figure windows, and then
%    plotting a Reviewed reconstruction to work on with this tool - EJR.
% 2. The tool allow you to select one rectangular region within the
%    Super-resolution reconstruction.
% 3. All localisations within this region are identified, and stored as
%    bridgedPosits and associated bridgedParams
% 4. Based on the box tracker code.
%% Part 0: Flow Control and Input Arguments
if verLessThan('matlab','8.4.0')
    reviewedParams = reviewer.results.reviewedSupResParams;
else
    if (strfind(figNewReconHandle.Name,'rejSupImage'))
        reviewedParams = reviewer.results.rejectedSupResParams;
    else
        reviewedParams = reviewer.results.reviewedSupResParams;
    end
end

reviewedPosits = [[reviewedParams.x_coord]' [reviewedParams.y_coord]'];
reconstructionScaleFactor = reviewer.results.reconstructionScaleFactor;

%%Identify user-boxed localisations
rect = getrect(figNewReconHandle);
% Above: user must use a UI to specify a region of the reconstuction.
bridgeCols = [rect(1), rect(1) + rect(3) ] ./ reconstructionScaleFactor;
bridgeRows = [rect(2), rect(2) + rect(4) ] ./ reconstructionScaleFactor;

selectedRows = ( (reviewedPosits(:,1) > bridgeRows(1)) & ...
    (reviewedPosits(:,1) < bridgeRows(2)) & ...
    (reviewedPosits(:,2) > bridgeCols(1)) & ...
    (reviewedPosits(:,2) < bridgeCols(2)) );

bridgedParams = reviewedParams( selectedRows )';
bridgedPosits = reviewedPosits( selectedRows, :);
% abs_sig_xy = abs([ bridgedParams.sig_x ]' ./ [ bridgedParams.sig_y ]' -1);

%% Display
% Note that Y-positions are plotted as negative row positions,
% to match image matrix orientation.

% Plot graphs
figSize = [1,100,1120,840];
figHistsHandle = figure(...
    'Name','Histogram Tracking',...
    'NumberTitle','off',...
    'OuterPosition', figSize ...
    );

% First Subplot:
subplot(2,2,1) % Use this if screen shape spoils the method below

% Histogram of counts associated with localisations, in 3x3 pixel centres
% Determine bar chart data:

I_vals = double([bridgedParams.I]);
bins = linspace(min(I_vals),max(I_vals),50);
bar_factor = 0.6;

[myNGood,bin_centers] = hist(I_vals,bins); % Get Hist data

hold on
h_bar = bar(bin_centers,[myNGood(:)],'r','stacked', 'BarWidth', bar_factor);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
title('Candidate Brightness (3x3)');
xlabel('Camera counts (in 3x3 centres)')
ylabel('Number of Candidates');

%% Histogram for first subplot;
% subplot(2,2,2)
% 
% bins_abs_sig_xy = 0.0125:0.025:1;
% hist(abs_sig_xy,bins_abs_sig_xy);
% title('Fit ellipticity histogram');
% xlabel('abs(sig_x/sig_y - 1)')
% set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');

%% Third supblot
subplot(2,2,3)

sig_x_vals = double([bridgedParams.sig_x]);
sig_y_vals = double([bridgedParams.sig_y]);
step = 0.1;
bar_factor = 0.4;
bins = 0.5:step:4;
[count_row_good,bin_centers] = hist(sig_x_vals,bins); % Get Hist data
% count_row_bad = hist(sig_x_vals(~qualityApprovedLocs),bins); % Get Hist data
count_col_good = hist(sig_y_vals,bins); % Get Hist data
% count_col_bad = hist(sig_y_vals(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_row = bar(bin_centers-bar_factor*step/2,[count_row_good(:)],'r','stacked', 'BarWidth', bar_factor);
h_col = bar(bin_centers+bar_factor*step/2,[count_col_good(:)],'b','stacked', 'BarWidth', bar_factor);
% set(h_row(2),'FaceColor',[0.4 0.4 0.4]);
% set(h_col(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
xlim([0.5-bar_factor*step 4+bar_factor*step])
title('Fitted Std Devs, Pixel Widths, row-direction is vertical');
ylabel('Number of Candidates');
if ~sum(count_row_good)==0 && ~sum(count_col_good)==0     % If there are red and blue bars
  legend([h_row(1) h_col(1)],'Row-direction','Col-direction'); % Then give a legend for them
end

%% Histogram for third subplot
% subplot(2,2,4)
% hist(residue_local);
% title('Residue histogram');
% xlabel('Residue')
% set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');

%% Fourth subplot
subplot(2,2,4);

bins = 0:5:150;
x_std_local = [ bridgedParams.x_std ];
y_std_local = [ bridgedParams.y_std ];

[count_row_good,bin_centers] = hist(x_std_local,bins); % Inter-Row direction
% count_row_bad = hist(x_std_vals(~qualityApprovedLocs),bins); % Inter-Row direction
count_col_good = hist(y_std_local,bins); % Inter-Column direction
% count_col_bad = hist(y_std_vals(~qualityApprovedLocs),bins); % Inter-Column direction

hold on
h_row = bar(bin_centers-1,[count_row_good(:)],'r','stacked', 'BarWidth', 0.4);
h_col = bar(bin_centers+1,[count_col_good(:)],'b','stacked', 'BarWidth', 0.4);
% set(h_row(2),'FaceColor',[0.4 0.4 0.4]);
% set(h_col(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
xlim([0 150])    % Set x-range to the interesting 0-150 nm region
title('Thompson Localisation Precisions (nm)');
ylabel('Number of Candidates');
if ~sum(count_row_good)==0 && ~sum(count_col_good)==0     % If there are red and blue bars
  legend([h_row(1) h_col(1)],'Row-direction','Col-direction'); % Then give a legend for them
end