function figHistsHandle = rainSTORM_histograms(ch_data,selected_recon,varargin)
% Copyright 2012. Refer to 00_license.txt for details.
% Plot Quality Control Data for Localisations

% 1. Read inputs
% Load localisation parameters from 'channel' structure.
SupResParams = ch_data.localization.results.SupResParams;
numberOfFrames = ch_data.localization.results.numberOfFrames;
qualityApprovedLocs = ch_data.reviewer{selected_recon, 1}.results.qualityApprovedLocs;
numberRejectedPerFrame = ch_data.reviewer{selected_recon, 1}.results.numberRejectedPerFrame;
numberAcceptedPerFrame = ch_data.reviewer{selected_recon, 1}.results.numberAcceptedPerFrame;

if isempty(varargin)
    text_figure_name = ['Histograms of Ch: ',ch_data.channel_name,...
    ', ',ch_data.reviewer{selected_recon}.recon_name];
else
    text_figure_name = varargin{1};
end

% 2. Plot graphs
figSize = [1,100,1120,840];
figHistsHandle = figure(...
    'Name',text_figure_name,...
    'NumberTitle','off',...
    'OuterPosition', figSize ...
    );


% First Subplot:
% subplot(2,2,1) % Use this if screen shape spoils the method below
subplot('Position', [0.07,0.56,0.4,0.4])

% Histogram of counts associated with localisations, in 3x3 pixel centres
% Determine bar chart data:

I_vals = double([SupResParams.I]);
bins = linspace(min(I_vals),max(I_vals),50);
bar_factor = 0.6;

[myNGood,bin_centers] = hist(I_vals(qualityApprovedLocs),bins); % Get Hist data
myNBad = hist(I_vals(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_bar = bar(bin_centers,[myNGood(:) myNBad(:)],'r','stacked', 'BarWidth', bar_factor);
set(h_bar(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
title('Candidate Brightness (3x3)');
xlabel('Camera counts (in 3x3 centres)')
ylabel('Number of Candidates');



% Second subplot:
% subplot(2,2,2)
subplot('Position', [0.57,0.56,0.4,0.4])
smooth_width = min(ceil(numberOfFrames/100),100);

to_plotRejected=zeros(1,numberOfFrames);
to_plotAccepted=zeros(1,numberOfFrames);
to_plotRejected(1:length(numberRejectedPerFrame)) = numberRejectedPerFrame;
to_plotAccepted(1:length(numberAcceptedPerFrame)) = numberAcceptedPerFrame;
to_plotRejectedSM = conv(to_plotRejected,ones(1,smooth_width),'same')./smooth_width;
to_plotAcceptedSM = conv(to_plotAccepted,ones(1,smooth_width),'same')./smooth_width;

plot(1:numberOfFrames, to_plotRejectedSM, 'r')
hold on
plot(1:numberOfFrames, to_plotAcceptedSM,'b')
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
title('Localisations per Frame (smoothed)');
xlabel('CCD frame number');
ylabel('Number per Frame');
legend('Rejected','Accepted','Location','Best');
hold off


% Third Plot
% Localisation diameters (fitted sigma values)
% New 22/3/2013 - CHECK for some bars "hiding" others, or other bugs
% subplot(2,2,3)
subplot('Position', [0.07,0.04,0.4,0.4])

sig_x_vals = double([SupResParams.sig_x]);
sig_y_vals = double([SupResParams.sig_y]);
step = 0.1;
bar_factor = 0.4;
bins = 0.5:step:4;
[count_row_good,bin_centers] = hist(sig_x_vals(qualityApprovedLocs),bins); % Get Hist data
count_row_bad = hist(sig_x_vals(~qualityApprovedLocs),bins); % Get Hist data
count_col_good = hist(sig_y_vals(qualityApprovedLocs),bins); % Get Hist data
count_col_bad = hist(sig_y_vals(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_row = bar(bin_centers-bar_factor*step/2,[count_row_good(:) count_row_bad(:)],'r','stacked', 'BarWidth', bar_factor);
h_col = bar(bin_centers+bar_factor*step/2,[count_col_good(:) count_col_bad(:)],'b','stacked', 'BarWidth', bar_factor);
set(h_row(2),'FaceColor',[0.4 0.4 0.4]);
set(h_col(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
xlim([0.5-bar_factor*step 4+bar_factor*step])
title('Fitted Std Devs, Pixel Widths, row-direction is vertical');
ylabel('Number of Candidates');
if ~sum(count_row_good)==0 && ~sum(count_col_good)==0     % If there are red and blue bars
  legend([h_row(1) h_col(1)],'Row-direction','Col-direction'); % Then give a legend for them
end


% Fourth Figure
% Thompson localisation precisions
% subplot(2,2,4)
subplot('Position', [0.57,0.04,0.4,0.4])

bins = 0:5:150;
x_std_vals=double([SupResParams.x_std]);
y_std_vals=double([SupResParams.y_std]);

[count_row_good,bin_centers] = hist(x_std_vals(qualityApprovedLocs),bins); % Inter-Row direction
count_row_bad = hist(x_std_vals(~qualityApprovedLocs),bins); % Inter-Row direction
count_col_good = hist(y_std_vals(qualityApprovedLocs),bins); % Inter-Column direction
count_col_bad = hist(y_std_vals(~qualityApprovedLocs),bins); % Inter-Column direction

hold on
h_row = bar(bin_centers-1,[count_row_good(:) count_row_bad(:)],'r','stacked', 'BarWidth', 0.4);
h_col = bar(bin_centers+1,[count_col_good(:) count_col_bad(:)],'b','stacked', 'BarWidth', 0.4);
set(h_row(2),'FaceColor',[0.4 0.4 0.4]);
set(h_col(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
xlim([0 150])    % Set x-range to the interesting 0-150 nm region
title('Thompson Localisation Precisions (nm)');
ylabel('Number of Candidates');
if ~sum(count_row_good)==0 && ~sum(count_col_good)==0     % If there are red and blue bars
  legend([h_row(1) h_col(1)],'Row-direction','Col-direction'); % Then give a legend for them
end

end