function figHistsHandle = rainSTORM_histograms_filters(ch_data,selected_recon,varargin)
% Copyright 2012. Refer to 00_license.txt for details.
% Plot Quality Control Data for Localisations

% 1. Read inputs
% Load localisation parameters from 'channel' structure.
SupResParams = ch_data.localization.results.SupResParams;
numberOfFrames = ch_data.localization.results.numberOfFrames;
qualityApprovedLocs = ch_data.reviewer{selected_recon, 1}.results.qualityApprovedLocs;
numberRejectedPerFrame = ch_data.reviewer{selected_recon, 1}.results.numberRejectedPerFrame;
numberAcceptedPerFrame = ch_data.reviewer{selected_recon, 1}.results.numberAcceptedPerFrame;

if isempty(varargin)
    text_figure_name = ['Histograms of Ch: ',ch_data.channel_name,...
    ', ',ch_data.reviewer{selected_recon}.recon_name];
else
    text_figure_name = varargin{1};
end

% 2. Plot graphs
figSize = [1,100,1120,840];
figHistsHandle = figure(...
    'Name',text_figure_name,...
    'NumberTitle','off',...
    'OuterPosition', figSize ...
    );


% First Subplot:
% subplot(2,2,1) % Use this if screen shape spoils the method below
subplot('Position', [0.07,0.56,0.4,0.4])

% Histogram that shows average brightness
% Determine bar chart data:

avg_brightness = double([SupResParams.avg_brightness]);
bins = linspace(min(avg_brightness),max(avg_brightness),50);
bar_factor = 0.6;

[myNGood,bin_centers] = hist(avg_brightness(qualityApprovedLocs),bins); % Get Hist data
myNBad = hist(avg_brightness(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_bar = bar(bin_centers,[myNGood(:) myNBad(:)],'r','stacked', 'BarWidth', bar_factor);
set(h_bar(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
title('Average Brightness');
xlabel('Brightness')
ylabel('Number of Candidates');



% Second subplot:
% subplot(2,2,2)
subplot('Position', [0.57,0.56,0.4,0.4])

roi = double([SupResParams.roi]);
bins = linspace(min(roi),max(roi),10);
bar_factor = 0.6;

[myNGood,bin_centers] = hist(roi(qualityApprovedLocs),bins); % Get Hist data
myNBad = hist(roi(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_bar = bar(bin_centers,[myNGood(:) myNBad(:)],'r','stacked', 'BarWidth', bar_factor);
set(h_bar(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
title('ROI');
xlabel('roi')
ylabel('Number of Candidates');



% Third Plot
% Localisation diameters (fitted sigma values)
% New 22/3/2013 - CHECK for some bars "hiding" others, or other bugs
% subplot(2,2,3)
subplot('Position', [0.07,0.04,0.4,0.4])

res_row_vals = double([SupResParams.res_Row]);
res_col_vals = double([SupResParams.res_Col]);
bins = linspace(min(res_row_vals),max(res_row_vals),50);
bar_factor = 0.4;
[count_row_good,bin_centers] = hist(res_row_vals(qualityApprovedLocs),bins); % Get Hist data
count_row_bad = hist(res_row_vals(~qualityApprovedLocs),bins); % Get Hist data
count_col_good = hist(res_col_vals(qualityApprovedLocs),bins); % Get Hist data
count_col_bad = hist(res_col_vals(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_row = bar(bin_centers-bar_factor*step/2,[count_row_good(:) count_row_bad(:)],'r','stacked', 'BarWidth', bar_factor);
h_col = bar(bin_centers+bar_factor*step/2,[count_col_good(:) count_col_bad(:)],'b','stacked', 'BarWidth', bar_factor);
set(h_row(2),'FaceColor',[0.4 0.4 0.4]);
set(h_col(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
xlim([0.5-bar_factor*step 4+bar_factor*step])
title('Fitted Residues, Pixel Widths, row-direction is vertical');
ylabel('Number of Candidates');
if ~sum(count_row_good)==0 && ~sum(count_col_good)==0     % If there are red and blue bars
  legend([h_row(1) h_col(1)],'Row-direction','Col-direction'); % Then give a legend for them
end


% Fourth Figure
% Thompson localisation precisions
% subplot(2,2,4)
subplot('Position', [0.57,0.04,0.4,0.4])

res = double([SupResParams.res]);
bins = linspace(min(res),max(res),10);
bar_factor = 0.6;

[myNGood,bin_centers] = hist(res(qualityApprovedLocs),bins); % Get Hist data
myNBad = hist(res(~qualityApprovedLocs),bins); % Get Hist data

hold on
h_bar = bar(bin_centers,[myNGood(:) myNBad(:)],'r','stacked', 'BarWidth', bar_factor);
set(h_bar(2),'FaceColor',[0.4 0.4 0.4]);
hold off
set(gca, 'FontSize', 12, 'FontWeight', 'bold', 'FontName', 'Times');
title('Residue');
xlabel('res')
ylabel('Number of Candidates');

end