function rainSTORM_report(ch_data,selected_recon)

% 0. Determine file name of the Localisation Microscopy data being handled
[pathstr, name, ext] = fileparts(ch_data.rawdata_mgr.fullfilename);
J = [fullfile(pathstr,name),'_',ch_data.reviewer{selected_recon,1}.recon_name];

% 1. Save a greyscale image of reconstructed frame
SupResIm = ch_data.reviewer{selected_recon,1}.results.SupResIm;
imwrite(uint16(SupResIm),[J,'-STORMdataImage.png']);

% 2. Get the on-screen Visualisation, maybe w/ colour, contrast, scalebar
figNewReconHandle = rainSTORM_display(ch_data.reviewer{selected_recon}.results.SupResIm,...         % SupResIm
        ch_data.reviewer{selected_recon}.results.reconstructionScaleFactor,...                      % linMag
        ch_data.reviewer{selected_recon}.settings,...                                               % flags to pass
        ch_data.reviewer{selected_recon}.recon_name);                                               % varargin
myIm = getframe(figNewReconHandle);
imwrite(myIm.cdata,[J,'-onScreenImage.png']);
close(figNewReconHandle)

% 2.0.1 Get the on-screen Visualisation, maybe w/ colour, contrast,
% scalebar + rejected
if (ch_data.reviewer{selected_recon,1}.settings.flagDebug)
figNewReconHandle = rainSTORM_display(ch_data.reviewer{selected_recon}.results.SupResIm,...         % SupResIm
        ch_data.reviewer{selected_recon}.results.reconstructionScaleFactor,...                      % linMag
        ch_data.reviewer{selected_recon}.settings,...                                               % flags to pass
        ch_data.reviewer{selected_recon}.recon_name);                                               % varargin
myIm = getframe(figNewReconHandle);
rejSupCo = ch_data.reviewer{selected_recon}.results.rejSupCo;
hold on 
plot(rejSupCo.y_coord,rejSupCo.x_coord, 'd','LineWidth',1,'Color','blue');
hold off
imwrite(myIm.cdata,[J,'-onpScreenImage.png']);
close(figNewReconHandle)
end

% 2.1 Get the rej-on-screen Visualisation, maybe w/ colour, contrast, scalebar
if (ch_data.reviewer{selected_recon,1}.settings.flagDebug)
figRejReconHandle = rainSTORM_display(ch_data.reviewer{selected_recon}.results.rejSupIm,...         % rejSupIm
        ch_data.reviewer{selected_recon}.results.reconstructionScaleFactor,...                      % linMag
        ch_data.reviewer{selected_recon}.settings,...                                               % flags to pass
        ch_data.reviewer{selected_recon}.recon_name);                                               % varargin
myIm = getframe(figRejReconHandle);
imwrite(myIm.cdata,[J,'-rejScreenImage.png']);
close(figRejReconHandle)
end

% 3. Save the Localisation data histograms.
figHistsHandle = rainSTORM_histograms(ch_data,selected_recon);
figHistsImage = getframe(figHistsHandle); 
imwrite(figHistsImage.cdata, [J,'-hists.png']);
close(figHistsHandle)

% 4. Save the sum of raw data: it approximatates a normal image
linMag = ch_data.reviewer{selected_recon}.results.reconstructionScaleFactor;% The linMag of the reconstructed image
sumFrame = ch_data.localization.results.sumFrame;
sumFrame = double(sumFrame);
sumFrame = sumFrame - min(sumFrame(:)); % Avoid washout of sum image
sumFrame = sumFrame/max(sumFrame(:));   % By saving after min-max scale
sumFrRsz = imresize(sumFrame,linMag,'nearest');
imwrite(sumFrRsz,[J,'-sum.png']);

% 5. Try saving some metadata about the reconstruction
try
    fid = fopen([J '-info.txt'], 'wt'); % "wt" indicates "Write Text"
    % 'w' mode needs Wordpad, since notepad dislikes 'w' newlines.
    % Use commas to separate columns, for easy import to Excel etc.
    fprintf(fid, '#Information about Reconstructed Image v3.1#\n\n' );
    fprintf(fid, 'Raw Image Data: %s \n', ch_data.rawdata_mgr.fullfilename);
    fprintf(fid, 'sizeOfCCDFrame: [%d %d]\n', ...
            ch_data.rawdata_mgr.myImInfo.sizeOfCCDFrame(1), ...
            ch_data.rawdata_mgr.myImInfo.sizeOfCCDFrame(2));
    fprintf(fid, 'Recon Name: %s \n', ch_data.reviewer{selected_recon,1}.recon_name);
    fprintf(fid, 'Date: %s \n\n', datestr(now, 'dd-mmm-yyyy  HH.MM.SS') );
    
    fprintf(fid, 'Precision Limit= 2*RMS Thompson precision (nm) ROW-axis: %f \n', ...
        ch_data.reviewer{selected_recon,1}.results.SparrowThompsonLimit(1));
    fprintf(fid, 'Precision Limit= 2*RMS Thompson precision (nm) COL-axis: %f \n', ...
        ch_data.reviewer{selected_recon,1}.results.SparrowThompsonLimit(2));
    fprintf(fid, 'Sparse Sampling Limit= 3*RMS Thompson precision (nm) ROW-axis: %f \n', ...
        1.5 * ch_data.reviewer{selected_recon,1}.results.SparrowThompsonLimit(1));
    fprintf(fid, 'Sparse Sampling Limit= 3*RMS Thompson precision (nm) COL-axis: %f \n', ...
        1.5 * ch_data.reviewer{selected_recon,1}.results.SparrowThompsonLimit(2));
    fprintf(fid, 'Mean precision estimate (row-direction nm): %f \n', ...
        ch_data.reviewer{selected_recon,1}.results.meanRevDeltaX(1));
    fprintf(fid, 'Mean precision estimate (col-direction nm): %f \n', ...
        ch_data.reviewer{selected_recon,1}.results.meanRevDeltaX(2));
    fprintf(fid, 'StDv precision estimate (row-direction nm): %f \n', ...
        ch_data.reviewer{selected_recon,1}.results.stdRevDeltaX(1));
    fprintf(fid, 'StDv precision estimate (col-direction nm): %f \n', ...
        ch_data.reviewer{selected_recon,1}.results.stdRevDeltaX(2));
    
    fprintf(fid, 'CCD pixel width on sample (nm): %i \n', ...
        ch_data.rawdata_mgr.myImInfo.pixelWidth);
    fprintf(fid, 'Super-resolution image pixel width (nm): %i \n', ...
        ch_data.rawdata_mgr.myImInfo.pixelWidth/ch_data.reviewer{selected_recon,1}.results.reconstructionScaleFactor);
    fprintf(fid, 'Counts per photon for calibration: %f \n', ...
        ch_data.rawdata_mgr.countsPerPhoton);
    fprintf(fid, 'Number of accepted localisations: %i \n', ...
        size(ch_data.reviewer{selected_recon,1}.results.reviewedSupResParams,2));
    fprintf(fid, 'Number of rejected candidates: %i \n', ...
        size(ch_data.localization.results.SupResParams,2)-size(ch_data.reviewer{selected_recon,1}.results.reviewedSupResParams,2));
    
    % Generate the info about the localization settings.
    fprintf(fid, '\n# Localization Settings: \n' );
     
        fprintf(fid, 'Localization Algorithm: %s \n', ...
            ch_data.localization.algo_name);
        fprintf(fid, '%s: %s \n', ...
            ch_data.localization.settings.ImageFilter.text, ...
            ch_data.localization.settings.ImageFilter.val);
        fprintf(fid, '%s: %i \n', ...
            ch_data.localization.settings.Thresh.text,...
            ch_data.localization.settings.Thresh.val);
        fprintf(fid, '%s: %i \n', ...
            ch_data.localization.settings.Thresh2.text,...
            ch_data.localization.settings.Thresh2.val);
        fprintf(fid, '%s: [%d %d]\n', ...
            ch_data.localization.settings.allowSig.text,...
            ch_data.localization.settings.allowSig.val(1),...
            ch_data.localization.settings.allowSig.val(2));
        fprintf(fid, '%s: %i \n', ...
            ch_data.localization.settings.allowX.text,...
            ch_data.localization.settings.allowX.val);
        fprintf(fid, '%s: %d \n', ...
            ch_data.localization.settings.initSig.text,...
            ch_data.localization.settings.initSig.val);
        fprintf(fid, '%s: %i \n', ...
            ch_data.localization.settings.maxIts.text,...
            ch_data.localization.settings.maxIts.val);
        fprintf(fid, '%s: %i \n', ...
            ch_data.localization.settings.prevSF.text,...
            ch_data.localization.settings.prevSF.val);
        fprintf(fid, '%s: %i \n', ...
            ch_data.localization.settings.rad.text,...
            ch_data.localization.settings.rad.val);
        fprintf(fid, '%s: %d \n', ...
            ch_data.localization.settings.tol.text,...
            ch_data.localization.settings.tol.val);
           
    % Generate the info about the reconstruction settings.
    fprintf(fid, '\n# Reconstruction Extended Settings: \n' );
    
        fprintf(fid, 'Reconstruction Algorithm: %s \n', ...
            ch_data.reviewer{selected_recon,1}.settings.recon_algo_name);
        fprintf(fid, '%s: %i \n', ...
            ch_data.reviewer{selected_recon,1}.settings.recon_algo_settings.linMag.text,...
            ch_data.reviewer{selected_recon,1}.settings.recon_algo_settings.linMag.val);
        fprintf(fid, 'Contrast Windowing: [%i %i] \n', ...
            ch_data.reviewer{selected_recon,1}.settings.windowing(1),...
            ch_data.reviewer{selected_recon,1}.settings.windowing(2));
                
    % Generate the info about the used filter data.
    fprintf(fid, '\n# Filter Data: \n' );
    
    num_of_filters = length(ch_data.reviewer{selected_recon,1}.settings.filters);
    for idx_filter = 1:num_of_filters
        filter_struct = ch_data.reviewer{selected_recon,1}.settings.filters{idx_filter,1};
        fprintf(fid, '\n%i/%i. filter: %s \n\n',...
            idx_filter,...
            num_of_filters,...
            filter_struct.filter_name);
        filter_settings_names = fieldnames(filter_struct.settings);
        num_of_params = length(filter_settings_names);
        for idx_param = 1:num_of_params
            fprintf(fid, '%s: %s \n',...
                filter_struct.settings.(filter_settings_names{idx_param}).text,...
                num2str(filter_struct.settings.(filter_settings_names{idx_param}).val,'% g'));
        end
    end
    fclose(fid);
catch myError
    warning('rainSTORM:Reviewer:saveInfoFailed','Info text file failed');
    try
        fclose(fid);
    catch myError
        warning('rainSTORM:Reviewer:saveInfoFailed','Text file may not be closed for writing.');
    end
end

% 6. If debug filters flag is enabled, save the created histogram.
if (ch_data.reviewer{selected_recon,1}.settings.flagDebug)
    imwrite(ch_data.reviewer{selected_recon,1}.results.filterOutputHists.cdata,[J,'-STORMfiltersDebug.png']);

end

end