function undrift = rainSTORM_undrift_auto(reviewedSupResParams)
 % function driftCorrection( data, beadAreas, outBase )
    if ~exist('outBase', 'var');
        outBase = 'image';
    end
    
    reviewedParams = reviewedSupResParams;
    
    x_coords = [reviewedParams.x_coord]';
    y_coords = [reviewedParams.y_coord]';
    x_stds = [reviewedParams.x_std]';
    y_stds = [reviewedParams.y_std]';
    intensities = [reviewedParams.I]';
    frameIndices = [reviewedParams.frame_idx]';
    


    nFrames = max( frameIndices );
    nLocs = length( x_coords );
    

    dirList = [];
    W = 1000;
    STEP = 1000;
    FRAMES = 1+W:STEP:nFrames-W;
    dir = [0.0 0.0];
    
    hWaitBar = waitbar(0, 'Drift Estimation: 0%' );
    
    for midFrame = FRAMES
        % Filter interval
        
        
        map = frameIndices >= midFrame-W & frameIndices < midFrame+W;
        x_coords_filtered = x_coords(map);
        y_coords_filtered = y_coords(map);
        intensities_filtered = intensities(map);
        frameIndices_filtered = frameIndices(map);
        x_stds_filtered = x_stds(map);
        y_stds_filtered = y_stds(map);
        

        dir = estimateLinearDrift( x_coords_filtered, y_coords_filtered, intensities_filtered, frameIndices_filtered, x_stds_filtered, y_stds_filtered, dir );

        fprintf( 1, 'dx: %.8f   dy:%.8f\n', dir(1), dir(2)  );
        
        dirList = [dirList; dir];
        
        perc = midFrame / nFrames;
        waitbar( perc, hWaitBar, sprintf('Drift Estimation: %d%%', round(perc*100) ) )
    end
    
    dirList = [ [0 0]; dirList; [0 0] ];
%     dirList = [ dirList(1,:); dirList; dirList(end,:) ];
    TList = [0 (FRAMES) nFrames-1];
    intType = 'linear'; % 'pchip';
    dxList = interp1( TList, dirList(:,1), 0:nFrames-1, intType, 'extrap' );
    dyList = interp1( TList, dirList(:,2), 0:nFrames-1, intType, 'extrap' );
    dx = cumsum( dxList );
    dy = cumsum( dyList );

    close( hWaitBar );
    
    undrift.params.dx = dx;
    undrift.params.dy = dy;

%     x_coords_c = num2cell( x_coords );
%     y_coords_c = num2cell( y_coords );
%     
%     [reviewedParams(:).x_coord] = deal(x_coords_c{:});
%     [reviewedParams(:).y_coord] = deal(y_coords_c{:});
    
end

function dir = estimateLinearDrift( x_coords, y_coords, intensities, frameIndices, x_stds, y_stds, x0 )
    precisions = max( [x_stds y_stds], [], 2 );

%     percent = 20000/length(x_coords); % 0.1;
%     WINDOW = 100;
%     
%     %% Filter intensities
%     map = false(size(precisions));
%     for i1 = 1:WINDOW:length(precisions)
%         i2 = min(i1-1+WINDOW, length(precisions));
%         
%         precs = precisions( i1:i2 );
%         sortedPrecs = sort( precs, 'ascend' ); ????????????? check!!!
%         precThreshold = sortedPrecs( ceil( (i2-i1+1)*percent ) );
%         
%         map( i1:i2 ) = precs >= precThreshold;
%     end
    
%     fprintf( 1, '%d points selected of %d\n', numel( find(map) ), length(x_coords) );
%     x_coords = x_coords(map);
%     y_coords = y_coords(map);
%     precisions =  precisions(map);
%     intensities = intensities(map);
%     frameIndices = frameIndices(map);
    
    DX = [];
    DY = [];
    DT = [];
    N = length( x_coords );

    mydist = @(v1, v2) bsxfun(@minus, v1', v2);
    
    points = 1000;
    nPairs = 500000;
    
    distThresh = 2;
    timeThresh = (max(frameIndices)-min(frameIndices)) * 0.5;
    
    BLOCKS = ceil(N / points );
    
% % % %     orderedIndices = randperm( N );
    [~, orderedIndices] = sort( precisions, 'ascend' );
tic
     for ii = 1:BLOCKS*2
         for jj = 1:BLOCKS+1
             i = ii - (jj-1);
             j = jj;
             
             if i < 1 || j < 1 || i > BLOCKS || j > BLOCKS || j > i
                 continue
             end
             
             ix1_1 = 1+(i-1)*points;
             ix1_2 = min(N, ix1_1+points-1);
             ix2_1 = 1+(j-1)*points;
             ix2_2 = min( [N; ix2_1+points-1; ix2_1+(ix1_2-ix1_1)] );
             
             indices = orderedIndices( ix1_1:ix1_2 );             
             indices2 = orderedIndices(ix2_1:ix2_2 );
           
             DX_ = mydist( x_coords( indices ), x_coords( indices2 ) );
             DY_ = mydist( y_coords( indices ), y_coords( indices2 ) );
             DT_ = mydist( frameIndices( indices ), frameIndices( indices2 ) );

             D = DX_.^2 + DY_.^2;
             
             map = D < distThresh^2 & abs(DT_) > timeThresh;
             
             DX = [DX; DX_(map)];
             DY = [DY; DY_(map)];
             DT = [DT; DT_(map)];
             
             if length(DX) > nPairs
                 break;
             end
         end
         if length(DX) > nPairs
             break;
         end
         
     end
         toc
        fprintf( 1, 'Selected point pairs: %d\n', length(DX) );


    options = optimoptions('fmincon');
%     options.OutputFcn = @callbackFunction;
    options.TolFun = 1e-15;
    options.DiffMaxChange = 1e-2;
    options.DiffMinChange = 1e-6;
    options.MaxIter = 200;
    options.Display = 'none'; % 'iter';

    lb = [-0.001 -0.001];% -0.0; 1.0];
    ub = [ 0.001  0.001];%  0.0; 1.0];
    
    if ~exist('x0', 'var')
        x0 = [ 0.0  0.0];%  0.0; 1.0];
    end
    
    
    
    sList = 2.^[0:-1:-2];
    for s = sList
        f = @(dir) fun(dir, s, DX, DY, DT );

        dir = fmincon( f, x0, [], [], [], [], lb, ub, [], options );
%         fprintf( 1, 'dx: %.8f   dy:%.8f\n', dir(1), dir(2)  );
        
        x0 = dir;
    end
end

function I = visDrift( dx, dy, nFrames )
    colors = jet(nFrames);
    
    fig = figure;
    scatter( dy, -dx, 12, colors, 'o');
    
    figImage = getframe(fig); 
    I = figImage.cdata;
    close(fig);
end

function stop = callbackFunction(dir, optimValues, state)
    stop = false;
    
    fprintf( 1, 'dx: %.8f   dy:%.8f\n', dir(1), dir(2) );
end

function r = fun( dir, s, DX, DY, DT )
    r = -sum(  exp( -( (DX - dir(1)*DT).^2  + (DY - dir(2)*DT).^2 )/ (2*s^2) )    );
end

