function startup

[rainSTORM_dir,~,~] = fileparts(mfilename('fullpath'));
addpath(genpath(rainSTORM_dir));
calibration_tool_def_dir = [rainSTORM_dir,filesep,'calibrations',filesep,'optoff'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check for Bio-Formats library and startup it.
% bfCheckJavaPath function clears global variables, thus this has to be at
% the begining.
status_Bio_Formats = 0;
if exist('bfCheckJavaPath.m','file') == 2
    [status_Bio_Formats, ~] = bfCheckJavaPath(1);
else
    disp('Bioformats Disabled');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global parameters
global rainSTORM_env

rainSTORM_env = struct(...
    'calibration_tool_def_dir',calibration_tool_def_dir,...
    'rainSTORM_dir',rainSTORM_dir,...
    'last_image_dir',rainSTORM_dir,...
    'parCompTool',0,...
    'parCompTool_NumWorkers',1,...
    'imgProcTool',0,...
    'is_tifflib_avail',false,...
    'is_Bio_Formats_library_avail',status_Bio_Formats...
    );

if exist('tifflib','file') == 3
    rainSTORM_env.is_tifflib_avail = true;
end

v = ver;
for k=1:length(v)
    if strfind(v(k).Name, 'Parallel Computing Toolbox')
        rainSTORM_env.parCompTool = str2num((sprintf( v(k).Version)));
    end
    if strfind(v(k).Name, 'Image Processing Toolbox')
        rainSTORM_env.imgProcTool = str2num((sprintf( v(k).Version)));
    end
    if strfind(v(k).Name, 'MATLAB')
        rainSTORM_env.matlabVer = str2num((sprintf( v(k).Version)));
    end
    if strfind(v(k).Name, 'Optimization Toolbox')
        rainSTORM_env.optTool = str2num((sprintf( v(k).Version)));
    end
end
if rainSTORM_env.imgProcTool <= 0
    error('There is no Image Processing Toolbox installed!');
end
if rainSTORM_env.optTool <= 0
    disp('There is no Optimization Toolbox installed!');
    disp('Auto drift correction will not work!');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start matlabpool if there is Parallel Computing Toolbox for R2014a or 
% greater and there is no active matlabpool.
if (rainSTORM_env.parCompTool >= 6) && (rainSTORM_env.matlabVer >= 8.4) && isempty(gcp('nocreate'))
    par_pool_data = gcp;
    rainSTORM_env.parCompTool_NumWorkers = par_pool_data.NumWorkers;
    disp('Parallel Computing Enabled');
else
    disp('Parallel Computing Disabled');
    disp('rainSTORM will run in Single-Threaded mode only');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Params Struct with default parameter settings.
params = rainSTORM_default_struct_params;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read back last image dir.
if exist([rainSTORM_env.rainSTORM_dir,filesep,'GUI',filesep,'last_image_dir.txt'], 'file')
    fid = fopen([rainSTORM_env.rainSTORM_dir,filesep,'GUI',filesep,'last_image_dir.txt']);
    last_image_dir  = fgetl(fid);
    fclose(fid);
    if isdir(last_image_dir)
        rainSTORM_env.last_image_dir = last_image_dir;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check version of matlab and give a notice about the difference in
% Reviewing tools
if verLessThan('matlab','8.4.0')
    disp('Matlab version R014a or earlier detected');
    error('Please upgrade');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Save params to the workspace
assignin('base', 'params', params);
disp('Startup complete!')
end

