function ch_data = rainSTORM_default_struct_ch_data

global rainSTORM_env

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rawdata_mgr
% This structure contains the parameters of the raw data.

rawdata_mgr=struct(...
    'fullfilename',[rainSTORM_env.rainSTORM_dir,'\test_images_by_testSTORM\images_magnitude\Offset_test01.tif'],... filename with path and extension
    'readmethod','nd2',... this specifies the method used to read the raw data
    'countsPerPhoton',100/67.8,... the conversion factor between read out values and the number of detected photons
    'myImInfo',struct('sizeOfCCDFrame',[128 128],'pixelWidth',160),... other image parameters, it might depend on the specific format.,...
    'optoff',[],... parameters of optical offset correction, if empty it is not applied after localization
    'transpose',false... Shall the raw image transposed ?
    );



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% localization
% The settings and the calculation results of the used localization algorithm.

algo_name = 'Least-Squares Gaussian 2D linear Bg';
% This structure differs for different localization algos probably.
settings = rainSTORM_default_settings_loc_algos(algo_name);

% All the outputs of the localization algorithm.
results=struct(...
    'SupResParams',[],...
    'numberOfFrames',[],...
    'numberOfLocs',[],...
    'sumFrame',[]...
    );

localization=struct(...
    'algo_name',algo_name,... The identifier of the used localization algorithm.
    'settings',settings,...
    'results',results...
    );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reviewer

% The parameters of box tracking feature.
boxtrack_params=struct(...
    'boxCols',[],...
    'boxRows',[],...
    'boxedColour',[],...
    'boxedParams',[],...
    'selectedLocs',[]...
    );

filter_name = 'rainSTORM algo 1-3';
filter_params=struct(...
    'filter_name',filter_name,...
    'settings',rainSTORM_default_settings_loc_filters(filter_name));

% Comment: the flagSB is for creating a scale bar in rewiever/rainSTORM_display.m
% Comment: the flagDebug is for filter debugging extra options
settings=struct(...
    'recon_algo_name','Simple Histogram',...
    'recon_algo_settings',rainSTORM_default_settings_recon_algo('Simple Histogram'),...
    'filters',{{filter_params;}},...
    'flagSB',true,...
    'windowing',[0 20],...
    'flagDebug',false,...
	'colormap',hot...
    );

results=struct(...    
    'SupResIm',[],...
    'rejSupCo',[],...
    'reviewedSupResParams',[],...
    'rejectedSupResParams',[],...
    'SparrowThompsonLimit',[],...
    'densestPoint',[],...
    'meanRevDeltaX',[],...
    'numberOfFits',[],...
    'qualityApprovedLocs',[],...
    'numberAcceptedPerFrame',[],...
    'numberRejectedPerFrame',[],...
    'stdRevDeltaX',[],...
    'reconstructionScaleFactor',[],...
    'recon_algo_outputs',[], ...
    'filterOutputHists',[] ...
    );

reviewer=struct(...
    'recon_name','Recon 1',...
    'settings',settings,...
    'results',results,...
    'error',[]...
    );

% reviewer2 = reviewer;
% reviewer2.recon_name = 'Recon 2';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% merge

ch_data = struct(...
    'channel_name','Channel',... % This string will denote the data on plots, figure headers, etc.
    'rawdata_mgr',rawdata_mgr,...
    'localization',localization,...
    'reviewer',{{reviewer;}});

end


